var five = require("../lib/johnny-five.js"),
    OpenNI = require("openni"),
    kinect;

five.Board().on("ready", function() {
  var defs, servos;

  // http://www.ranchbots.com/robot_arm/images/arm_diagram.jpg
  defs = [
    // Pivot/Rotator
    { id: "rotator", pin:  6, range: [  10, 170 ], startAt: 90 },
    // Shoulder
    { id: "shoulder", pin:  9, range: [ 20, 150 ], startAt: 90 },
    // Elbow
    { id: "elbow", pin: 10, range: [  10, 120 ], startAt: 90 },
    // Wrist
    { id: "wrist", pin: 11, range: [  10, 170 ], startAt: 40 },
    // Grip
    { id: "claw", pin: 12, range: [  10, 170 ], startAt: 0 }
  ];

  // Reduce the des array into an object of servo instances,
  // where the servo id is the property name
  servos = defs.reduce(function( accum, def ) {
    return (accum[ def.id ] = five.Servo( def )) && accum;
  }, {});

  // Make servos accessible by name in the REPL
  this.repl.inject( servos );

  // Initialize the OpenNI/Kinect bindings API
  kinect = OpenNI();

  if ( kinect ) {
    // Eventually we'll use more of the available skeletal points,
    // but for now, the elbow is the only point we need.
    //
    [
      // "right_collar",
      // "right_shoulder",
      "right_elbow",
      // "right_wrist",
      // "right_hand",
      // "right_fingertip"
    ].forEach(function( joint ) {
      // x-right, y-downwards, z-forward
      kinect.on( joint, motion );
    });
  }

  function motion( user, x, y, z ) {
    // x-right, y-downwards, z-forward
    var name = joint.replace("right_", "");

    // Currently, we'll only receive "elbow" events, but eventually we'll
    // need to account for several possible skeletal points
    if ( name === "elbow" ) {
      servos.shoulder.move( five.Fn.scale( y, 250, 70, 90, 150 ) );
      servos.elbow.move( five.Fn.scale( x, 80, 170, 20, 120 ) );
    }
  }
});
